# Copyright (C) 2018-2019  Lars Wirzenius
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import ick2


class TransactionalResource:

    def __init__(self, token, state, kind, name):
        self.token = token
        self.state = state
        self.kind = kind
        self.name = name
        if state.has_resource(self.token, kind, name):
            self.new = False
            self.resource = state.get_resource(self.token, kind, name)
        else:
            self.new = True
            self.resource = ick2.resource_from_dict({})

        methods = [
            'as_dict',
            '__getitem__',
            '__setitem__',
            '__contains__',
            '__len__',
        ]
        for method in methods:
            setattr(self, method, getattr(self.resource, method))

    def __enter__(self):
        return self.resource

    def __exit__(self, exc_type, value, traceback):
        if exc_type is None:
            if self.new:
                self.state.write_resource(
                    self.token, self.kind, self.name, self.resource)
            else:
                self.state.update_resource(
                    self.token, self.kind, self.name, self.resource)


class TransactionalState:

    def __init__(self, state):
        self.state = state

    def new(self, token, kind, name):
        return TransactionalResource(token, self.state, kind, name)

    def modify(self, token, kind, name):
        if not self.state.has_resource(token, kind, name):
            raise ick2.NotFound(kind=kind, name=name)
        return TransactionalResource(token, self.state, kind, name)

    def has_resource(self, token, kind, name):
        return self.state.has_resource(token, kind, name)

    def get_resource(self, token, kind, name):
        return self.state.get_resource(token, kind, name)

    def get_resources(self, token, kind):
        return self.state.get_resources(token, kind)

    def remove_resource(self, token, kind, name):
        self.state.remove_resource(token, kind, name)
